# $Id$

package ObsData::Archive::Tar;

use strict;
use warnings;
use File::Copy;
use File::Temp qw/ tempfile tempdir /;
use Cwd;

our @ISA = qw(ObsData::Archive);

sub new {
    my ($class, $options) = @_;

    if (!$options->{uncomp}) {
        for ($options->{archive}) {
            /\.(Z|gz)$/ and $options->{uncomp} = 'gzip -dc';
            /\.bz2$/ and $options->{uncomp} = 'bzip2 -dc';
        }
    }
    $options->{uncomp} ||= 'cat';
    bless($options, $class);
}

sub DESTROY {
    my ($self) = @_;
}

sub ls {
    my ($self) = @_;
    open(my $htar, '-|', "$self->{uncomp} '$self->{archive}' | tar tf -") or
        return undef;
    my @list;
    while(<$htar>) {
        chomp;
        push(@list, $_);
    }
    close($htar);
    return @list;
}

sub extract {
    my ($self, $file, $dest) = @_;

    my $tempdir = tempdir();
    my $here = getcwd();
    my $abs_path = Cwd::abs_path($self->{archive});
    my $abs_dest = Cwd::abs_path($dest);
    
    chdir($tempdir);

    system("$self->{uncomp} '$abs_path' | tar xf - '$file'") and do {
        chdir($here);
        return undef;
    };
    
    if ($dest) {
        open(my $fh, '>', $abs_dest) or return undef;
        if(!copy("$tempdir/$file", $fh)) {
            unlink($abs_dest);
            chdir($here);
            return undef;
        }
        close($fh);
        unlink("$tempdir/$file");
        chdir($here);
        return $dest;
    } else {
        chdir($here);
        return "$tempdir/$file";
    }
}

